# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['FileArgs', 'File']

@pulumi.input_type
class FileArgs:
    def __init__(__self__, *,
                 datastore_id: pulumi.Input[str],
                 node_name: pulumi.Input[str],
                 content_type: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 source_file: Optional[pulumi.Input['FileSourceFileArgs']] = None,
                 source_raw: Optional[pulumi.Input['FileSourceRawArgs']] = None,
                 timeout_upload: Optional[pulumi.Input[int]] = None):
        """
        The set of arguments for constructing a File resource.
        :param pulumi.Input[str] datastore_id: The datastore id.
        :param pulumi.Input[str] node_name: The node name.
        :param pulumi.Input[str] content_type: The content type.
        :param pulumi.Input[bool] overwrite: Whether to overwrite an existing file (defaults to
               `true`).
        :param pulumi.Input['FileSourceFileArgs'] source_file: The source file (conflicts with `source_raw`).
        :param pulumi.Input['FileSourceRawArgs'] source_raw: The raw source (conflicts with `source_file`).
        :param pulumi.Input[int] timeout_upload: Timeout for uploading ISO/VSTMPL files in
               seconds (defaults to 1800).
        """
        FileArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            datastore_id=datastore_id,
            node_name=node_name,
            content_type=content_type,
            overwrite=overwrite,
            source_file=source_file,
            source_raw=source_raw,
            timeout_upload=timeout_upload,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             datastore_id: pulumi.Input[str],
             node_name: pulumi.Input[str],
             content_type: Optional[pulumi.Input[str]] = None,
             overwrite: Optional[pulumi.Input[bool]] = None,
             source_file: Optional[pulumi.Input['FileSourceFileArgs']] = None,
             source_raw: Optional[pulumi.Input['FileSourceRawArgs']] = None,
             timeout_upload: Optional[pulumi.Input[int]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'datastoreId' in kwargs:
            datastore_id = kwargs['datastoreId']
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']
        if 'contentType' in kwargs:
            content_type = kwargs['contentType']
        if 'sourceFile' in kwargs:
            source_file = kwargs['sourceFile']
        if 'sourceRaw' in kwargs:
            source_raw = kwargs['sourceRaw']
        if 'timeoutUpload' in kwargs:
            timeout_upload = kwargs['timeoutUpload']

        _setter("datastore_id", datastore_id)
        _setter("node_name", node_name)
        if content_type is not None:
            _setter("content_type", content_type)
        if overwrite is not None:
            _setter("overwrite", overwrite)
        if source_file is not None:
            _setter("source_file", source_file)
        if source_raw is not None:
            _setter("source_raw", source_raw)
        if timeout_upload is not None:
            _setter("timeout_upload", timeout_upload)

    @property
    @pulumi.getter(name="datastoreId")
    def datastore_id(self) -> pulumi.Input[str]:
        """
        The datastore id.
        """
        return pulumi.get(self, "datastore_id")

    @datastore_id.setter
    def datastore_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "datastore_id", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Input[str]:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter(name="contentType")
    def content_type(self) -> Optional[pulumi.Input[str]]:
        """
        The content type.
        """
        return pulumi.get(self, "content_type")

    @content_type.setter
    def content_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "content_type", value)

    @property
    @pulumi.getter
    def overwrite(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to overwrite an existing file (defaults to
        `true`).
        """
        return pulumi.get(self, "overwrite")

    @overwrite.setter
    def overwrite(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "overwrite", value)

    @property
    @pulumi.getter(name="sourceFile")
    def source_file(self) -> Optional[pulumi.Input['FileSourceFileArgs']]:
        """
        The source file (conflicts with `source_raw`).
        """
        return pulumi.get(self, "source_file")

    @source_file.setter
    def source_file(self, value: Optional[pulumi.Input['FileSourceFileArgs']]):
        pulumi.set(self, "source_file", value)

    @property
    @pulumi.getter(name="sourceRaw")
    def source_raw(self) -> Optional[pulumi.Input['FileSourceRawArgs']]:
        """
        The raw source (conflicts with `source_file`).
        """
        return pulumi.get(self, "source_raw")

    @source_raw.setter
    def source_raw(self, value: Optional[pulumi.Input['FileSourceRawArgs']]):
        pulumi.set(self, "source_raw", value)

    @property
    @pulumi.getter(name="timeoutUpload")
    def timeout_upload(self) -> Optional[pulumi.Input[int]]:
        """
        Timeout for uploading ISO/VSTMPL files in
        seconds (defaults to 1800).
        """
        return pulumi.get(self, "timeout_upload")

    @timeout_upload.setter
    def timeout_upload(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "timeout_upload", value)


@pulumi.input_type
class _FileState:
    def __init__(__self__, *,
                 content_type: Optional[pulumi.Input[str]] = None,
                 datastore_id: Optional[pulumi.Input[str]] = None,
                 file_modification_date: Optional[pulumi.Input[str]] = None,
                 file_name: Optional[pulumi.Input[str]] = None,
                 file_size: Optional[pulumi.Input[int]] = None,
                 file_tag: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 source_file: Optional[pulumi.Input['FileSourceFileArgs']] = None,
                 source_raw: Optional[pulumi.Input['FileSourceRawArgs']] = None,
                 timeout_upload: Optional[pulumi.Input[int]] = None):
        """
        Input properties used for looking up and filtering File resources.
        :param pulumi.Input[str] content_type: The content type.
        :param pulumi.Input[str] datastore_id: The datastore id.
        :param pulumi.Input[str] file_modification_date: The file modification date (RFC 3339).
        :param pulumi.Input[str] file_name: The file name.
        :param pulumi.Input[int] file_size: The file size in bytes.
        :param pulumi.Input[str] file_tag: The file tag.
        :param pulumi.Input[str] node_name: The node name.
        :param pulumi.Input[bool] overwrite: Whether to overwrite an existing file (defaults to
               `true`).
        :param pulumi.Input['FileSourceFileArgs'] source_file: The source file (conflicts with `source_raw`).
        :param pulumi.Input['FileSourceRawArgs'] source_raw: The raw source (conflicts with `source_file`).
        :param pulumi.Input[int] timeout_upload: Timeout for uploading ISO/VSTMPL files in
               seconds (defaults to 1800).
        """
        _FileState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            content_type=content_type,
            datastore_id=datastore_id,
            file_modification_date=file_modification_date,
            file_name=file_name,
            file_size=file_size,
            file_tag=file_tag,
            node_name=node_name,
            overwrite=overwrite,
            source_file=source_file,
            source_raw=source_raw,
            timeout_upload=timeout_upload,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             content_type: Optional[pulumi.Input[str]] = None,
             datastore_id: Optional[pulumi.Input[str]] = None,
             file_modification_date: Optional[pulumi.Input[str]] = None,
             file_name: Optional[pulumi.Input[str]] = None,
             file_size: Optional[pulumi.Input[int]] = None,
             file_tag: Optional[pulumi.Input[str]] = None,
             node_name: Optional[pulumi.Input[str]] = None,
             overwrite: Optional[pulumi.Input[bool]] = None,
             source_file: Optional[pulumi.Input['FileSourceFileArgs']] = None,
             source_raw: Optional[pulumi.Input['FileSourceRawArgs']] = None,
             timeout_upload: Optional[pulumi.Input[int]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'contentType' in kwargs:
            content_type = kwargs['contentType']
        if 'datastoreId' in kwargs:
            datastore_id = kwargs['datastoreId']
        if 'fileModificationDate' in kwargs:
            file_modification_date = kwargs['fileModificationDate']
        if 'fileName' in kwargs:
            file_name = kwargs['fileName']
        if 'fileSize' in kwargs:
            file_size = kwargs['fileSize']
        if 'fileTag' in kwargs:
            file_tag = kwargs['fileTag']
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']
        if 'sourceFile' in kwargs:
            source_file = kwargs['sourceFile']
        if 'sourceRaw' in kwargs:
            source_raw = kwargs['sourceRaw']
        if 'timeoutUpload' in kwargs:
            timeout_upload = kwargs['timeoutUpload']

        if content_type is not None:
            _setter("content_type", content_type)
        if datastore_id is not None:
            _setter("datastore_id", datastore_id)
        if file_modification_date is not None:
            _setter("file_modification_date", file_modification_date)
        if file_name is not None:
            _setter("file_name", file_name)
        if file_size is not None:
            _setter("file_size", file_size)
        if file_tag is not None:
            _setter("file_tag", file_tag)
        if node_name is not None:
            _setter("node_name", node_name)
        if overwrite is not None:
            _setter("overwrite", overwrite)
        if source_file is not None:
            _setter("source_file", source_file)
        if source_raw is not None:
            _setter("source_raw", source_raw)
        if timeout_upload is not None:
            _setter("timeout_upload", timeout_upload)

    @property
    @pulumi.getter(name="contentType")
    def content_type(self) -> Optional[pulumi.Input[str]]:
        """
        The content type.
        """
        return pulumi.get(self, "content_type")

    @content_type.setter
    def content_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "content_type", value)

    @property
    @pulumi.getter(name="datastoreId")
    def datastore_id(self) -> Optional[pulumi.Input[str]]:
        """
        The datastore id.
        """
        return pulumi.get(self, "datastore_id")

    @datastore_id.setter
    def datastore_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "datastore_id", value)

    @property
    @pulumi.getter(name="fileModificationDate")
    def file_modification_date(self) -> Optional[pulumi.Input[str]]:
        """
        The file modification date (RFC 3339).
        """
        return pulumi.get(self, "file_modification_date")

    @file_modification_date.setter
    def file_modification_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "file_modification_date", value)

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> Optional[pulumi.Input[str]]:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @file_name.setter
    def file_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "file_name", value)

    @property
    @pulumi.getter(name="fileSize")
    def file_size(self) -> Optional[pulumi.Input[int]]:
        """
        The file size in bytes.
        """
        return pulumi.get(self, "file_size")

    @file_size.setter
    def file_size(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "file_size", value)

    @property
    @pulumi.getter(name="fileTag")
    def file_tag(self) -> Optional[pulumi.Input[str]]:
        """
        The file tag.
        """
        return pulumi.get(self, "file_tag")

    @file_tag.setter
    def file_tag(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "file_tag", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter
    def overwrite(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to overwrite an existing file (defaults to
        `true`).
        """
        return pulumi.get(self, "overwrite")

    @overwrite.setter
    def overwrite(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "overwrite", value)

    @property
    @pulumi.getter(name="sourceFile")
    def source_file(self) -> Optional[pulumi.Input['FileSourceFileArgs']]:
        """
        The source file (conflicts with `source_raw`).
        """
        return pulumi.get(self, "source_file")

    @source_file.setter
    def source_file(self, value: Optional[pulumi.Input['FileSourceFileArgs']]):
        pulumi.set(self, "source_file", value)

    @property
    @pulumi.getter(name="sourceRaw")
    def source_raw(self) -> Optional[pulumi.Input['FileSourceRawArgs']]:
        """
        The raw source (conflicts with `source_file`).
        """
        return pulumi.get(self, "source_raw")

    @source_raw.setter
    def source_raw(self, value: Optional[pulumi.Input['FileSourceRawArgs']]):
        pulumi.set(self, "source_raw", value)

    @property
    @pulumi.getter(name="timeoutUpload")
    def timeout_upload(self) -> Optional[pulumi.Input[int]]:
        """
        Timeout for uploading ISO/VSTMPL files in
        seconds (defaults to 1800).
        """
        return pulumi.get(self, "timeout_upload")

    @timeout_upload.setter
    def timeout_upload(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "timeout_upload", value)


class File(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 content_type: Optional[pulumi.Input[str]] = None,
                 datastore_id: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 source_file: Optional[pulumi.Input[pulumi.InputType['FileSourceFileArgs']]] = None,
                 source_raw: Optional[pulumi.Input[pulumi.InputType['FileSourceRawArgs']]] = None,
                 timeout_upload: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        """
        Manages a file.

        ## Important Notes

        The Proxmox VE API endpoint for file uploads does not support chunked transfer
        encoding, which means that we must first store the source file as a temporary
        file locally before uploading it.

        You must ensure that you have at least `Size-in-MB * 2 + 1` MB of storage space
        available (twice the size plus overhead because a multipart payload needs to be
        created as another temporary file).

        By default, if the specified file already exists, the resource will
        unconditionally replace it and take ownership of the resource. On destruction,
        the file will be deleted as if it did not exist before. If you want to prevent
        the resource from replacing the file, set `overwrite` to `false`.

        ## Import

        Instances can be imported using the `node_name`, `datastore_id`, `content_type` and the `file_name` in the following format<node_name>:<datastore_id>/<content_type>/<file_name> Examplebash

        ```sh
         $ pulumi import proxmoxve:Storage/file:File cloud_config pve/local:snippets/example.cloud-config.yaml
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] content_type: The content type.
        :param pulumi.Input[str] datastore_id: The datastore id.
        :param pulumi.Input[str] node_name: The node name.
        :param pulumi.Input[bool] overwrite: Whether to overwrite an existing file (defaults to
               `true`).
        :param pulumi.Input[pulumi.InputType['FileSourceFileArgs']] source_file: The source file (conflicts with `source_raw`).
        :param pulumi.Input[pulumi.InputType['FileSourceRawArgs']] source_raw: The raw source (conflicts with `source_file`).
        :param pulumi.Input[int] timeout_upload: Timeout for uploading ISO/VSTMPL files in
               seconds (defaults to 1800).
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: FileArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a file.

        ## Important Notes

        The Proxmox VE API endpoint for file uploads does not support chunked transfer
        encoding, which means that we must first store the source file as a temporary
        file locally before uploading it.

        You must ensure that you have at least `Size-in-MB * 2 + 1` MB of storage space
        available (twice the size plus overhead because a multipart payload needs to be
        created as another temporary file).

        By default, if the specified file already exists, the resource will
        unconditionally replace it and take ownership of the resource. On destruction,
        the file will be deleted as if it did not exist before. If you want to prevent
        the resource from replacing the file, set `overwrite` to `false`.

        ## Import

        Instances can be imported using the `node_name`, `datastore_id`, `content_type` and the `file_name` in the following format<node_name>:<datastore_id>/<content_type>/<file_name> Examplebash

        ```sh
         $ pulumi import proxmoxve:Storage/file:File cloud_config pve/local:snippets/example.cloud-config.yaml
        ```

        :param str resource_name: The name of the resource.
        :param FileArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(FileArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            FileArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 content_type: Optional[pulumi.Input[str]] = None,
                 datastore_id: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 source_file: Optional[pulumi.Input[pulumi.InputType['FileSourceFileArgs']]] = None,
                 source_raw: Optional[pulumi.Input[pulumi.InputType['FileSourceRawArgs']]] = None,
                 timeout_upload: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = FileArgs.__new__(FileArgs)

            __props__.__dict__["content_type"] = content_type
            if datastore_id is None and not opts.urn:
                raise TypeError("Missing required property 'datastore_id'")
            __props__.__dict__["datastore_id"] = datastore_id
            if node_name is None and not opts.urn:
                raise TypeError("Missing required property 'node_name'")
            __props__.__dict__["node_name"] = node_name
            __props__.__dict__["overwrite"] = overwrite
            if source_file is not None and not isinstance(source_file, FileSourceFileArgs):
                source_file = source_file or {}
                def _setter(key, value):
                    source_file[key] = value
                FileSourceFileArgs._configure(_setter, **source_file)
            __props__.__dict__["source_file"] = source_file
            if source_raw is not None and not isinstance(source_raw, FileSourceRawArgs):
                source_raw = source_raw or {}
                def _setter(key, value):
                    source_raw[key] = value
                FileSourceRawArgs._configure(_setter, **source_raw)
            __props__.__dict__["source_raw"] = source_raw
            __props__.__dict__["timeout_upload"] = timeout_upload
            __props__.__dict__["file_modification_date"] = None
            __props__.__dict__["file_name"] = None
            __props__.__dict__["file_size"] = None
            __props__.__dict__["file_tag"] = None
        super(File, __self__).__init__(
            'proxmoxve:Storage/file:File',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            content_type: Optional[pulumi.Input[str]] = None,
            datastore_id: Optional[pulumi.Input[str]] = None,
            file_modification_date: Optional[pulumi.Input[str]] = None,
            file_name: Optional[pulumi.Input[str]] = None,
            file_size: Optional[pulumi.Input[int]] = None,
            file_tag: Optional[pulumi.Input[str]] = None,
            node_name: Optional[pulumi.Input[str]] = None,
            overwrite: Optional[pulumi.Input[bool]] = None,
            source_file: Optional[pulumi.Input[pulumi.InputType['FileSourceFileArgs']]] = None,
            source_raw: Optional[pulumi.Input[pulumi.InputType['FileSourceRawArgs']]] = None,
            timeout_upload: Optional[pulumi.Input[int]] = None) -> 'File':
        """
        Get an existing File resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] content_type: The content type.
        :param pulumi.Input[str] datastore_id: The datastore id.
        :param pulumi.Input[str] file_modification_date: The file modification date (RFC 3339).
        :param pulumi.Input[str] file_name: The file name.
        :param pulumi.Input[int] file_size: The file size in bytes.
        :param pulumi.Input[str] file_tag: The file tag.
        :param pulumi.Input[str] node_name: The node name.
        :param pulumi.Input[bool] overwrite: Whether to overwrite an existing file (defaults to
               `true`).
        :param pulumi.Input[pulumi.InputType['FileSourceFileArgs']] source_file: The source file (conflicts with `source_raw`).
        :param pulumi.Input[pulumi.InputType['FileSourceRawArgs']] source_raw: The raw source (conflicts with `source_file`).
        :param pulumi.Input[int] timeout_upload: Timeout for uploading ISO/VSTMPL files in
               seconds (defaults to 1800).
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _FileState.__new__(_FileState)

        __props__.__dict__["content_type"] = content_type
        __props__.__dict__["datastore_id"] = datastore_id
        __props__.__dict__["file_modification_date"] = file_modification_date
        __props__.__dict__["file_name"] = file_name
        __props__.__dict__["file_size"] = file_size
        __props__.__dict__["file_tag"] = file_tag
        __props__.__dict__["node_name"] = node_name
        __props__.__dict__["overwrite"] = overwrite
        __props__.__dict__["source_file"] = source_file
        __props__.__dict__["source_raw"] = source_raw
        __props__.__dict__["timeout_upload"] = timeout_upload
        return File(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="contentType")
    def content_type(self) -> pulumi.Output[str]:
        """
        The content type.
        """
        return pulumi.get(self, "content_type")

    @property
    @pulumi.getter(name="datastoreId")
    def datastore_id(self) -> pulumi.Output[str]:
        """
        The datastore id.
        """
        return pulumi.get(self, "datastore_id")

    @property
    @pulumi.getter(name="fileModificationDate")
    def file_modification_date(self) -> pulumi.Output[str]:
        """
        The file modification date (RFC 3339).
        """
        return pulumi.get(self, "file_modification_date")

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> pulumi.Output[str]:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @property
    @pulumi.getter(name="fileSize")
    def file_size(self) -> pulumi.Output[int]:
        """
        The file size in bytes.
        """
        return pulumi.get(self, "file_size")

    @property
    @pulumi.getter(name="fileTag")
    def file_tag(self) -> pulumi.Output[str]:
        """
        The file tag.
        """
        return pulumi.get(self, "file_tag")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Output[str]:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def overwrite(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether to overwrite an existing file (defaults to
        `true`).
        """
        return pulumi.get(self, "overwrite")

    @property
    @pulumi.getter(name="sourceFile")
    def source_file(self) -> pulumi.Output[Optional['outputs.FileSourceFile']]:
        """
        The source file (conflicts with `source_raw`).
        """
        return pulumi.get(self, "source_file")

    @property
    @pulumi.getter(name="sourceRaw")
    def source_raw(self) -> pulumi.Output[Optional['outputs.FileSourceRaw']]:
        """
        The raw source (conflicts with `source_file`).
        """
        return pulumi.get(self, "source_raw")

    @property
    @pulumi.getter(name="timeoutUpload")
    def timeout_upload(self) -> pulumi.Output[Optional[int]]:
        """
        Timeout for uploading ISO/VSTMPL files in
        seconds (defaults to 1800).
        """
        return pulumi.get(self, "timeout_upload")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetRolesResult',
    'AwaitableGetRolesResult',
    'get_roles',
    'get_roles_output',
]

@pulumi.output_type
class GetRolesResult:
    """
    A collection of values returned by getRoles.
    """
    def __init__(__self__, id=None, privileges=None, role_ids=None, specials=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if privileges and not isinstance(privileges, list):
            raise TypeError("Expected argument 'privileges' to be a list")
        pulumi.set(__self__, "privileges", privileges)
        if role_ids and not isinstance(role_ids, list):
            raise TypeError("Expected argument 'role_ids' to be a list")
        pulumi.set(__self__, "role_ids", role_ids)
        if specials and not isinstance(specials, list):
            raise TypeError("Expected argument 'specials' to be a list")
        pulumi.set(__self__, "specials", specials)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def privileges(self) -> Sequence[Sequence[str]]:
        """
        The role privileges.
        """
        return pulumi.get(self, "privileges")

    @property
    @pulumi.getter(name="roleIds")
    def role_ids(self) -> Sequence[str]:
        """
        The role identifiers.
        """
        return pulumi.get(self, "role_ids")

    @property
    @pulumi.getter
    def specials(self) -> Sequence[bool]:
        """
        Whether the role is special (built-in).
        """
        return pulumi.get(self, "specials")


class AwaitableGetRolesResult(GetRolesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetRolesResult(
            id=self.id,
            privileges=self.privileges,
            role_ids=self.role_ids,
            specials=self.specials)


def get_roles(opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetRolesResult:
    """
    Retrieves information about all the available roles.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    available_roles = proxmoxve.Permission.get_roles()
    ```
    """
    __args__ = dict()
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:Permission/getRoles:getRoles', __args__, opts=opts, typ=GetRolesResult).value

    return AwaitableGetRolesResult(
        id=pulumi.get(__ret__, 'id'),
        privileges=pulumi.get(__ret__, 'privileges'),
        role_ids=pulumi.get(__ret__, 'role_ids'),
        specials=pulumi.get(__ret__, 'specials'))


@_utilities.lift_output_func(get_roles)
def get_roles_output(opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetRolesResult]:
    """
    Retrieves information about all the available roles.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    available_roles = proxmoxve.Permission.get_roles()
    ```
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'GetDNSResult',
    'AwaitableGetDNSResult',
    'get_dns',
    'get_dns_output',
]

@pulumi.output_type
class GetDNSResult:
    """
    A collection of values returned by getDNS.
    """
    def __init__(__self__, domain=None, id=None, node_name=None, servers=None):
        if domain and not isinstance(domain, str):
            raise TypeError("Expected argument 'domain' to be a str")
        pulumi.set(__self__, "domain", domain)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if node_name and not isinstance(node_name, str):
            raise TypeError("Expected argument 'node_name' to be a str")
        pulumi.set(__self__, "node_name", node_name)
        if servers and not isinstance(servers, list):
            raise TypeError("Expected argument 'servers' to be a list")
        pulumi.set(__self__, "servers", servers)

    @property
    @pulumi.getter
    def domain(self) -> str:
        """
        The DNS search domain.
        """
        return pulumi.get(self, "domain")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> str:
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def servers(self) -> Sequence[str]:
        """
        The DNS servers.
        """
        return pulumi.get(self, "servers")


class AwaitableGetDNSResult(GetDNSResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDNSResult(
            domain=self.domain,
            id=self.id,
            node_name=self.node_name,
            servers=self.servers)


def get_dns(node_name: Optional[str] = None,
            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetDNSResult:
    """
    Retrieves the DNS configuration for a specific node.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    first_node = proxmoxve.get_dns(node_name="first-node")
    ```


    :param str node_name: A node name.
    """
    __args__ = dict()
    __args__['nodeName'] = node_name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:index/getDNS:getDNS', __args__, opts=opts, typ=GetDNSResult).value

    return AwaitableGetDNSResult(
        domain=pulumi.get(__ret__, 'domain'),
        id=pulumi.get(__ret__, 'id'),
        node_name=pulumi.get(__ret__, 'node_name'),
        servers=pulumi.get(__ret__, 'servers'))


@_utilities.lift_output_func(get_dns)
def get_dns_output(node_name: Optional[pulumi.Input[str]] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetDNSResult]:
    """
    Retrieves the DNS configuration for a specific node.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    first_node = proxmoxve.get_dns(node_name="first-node")
    ```


    :param str node_name: A node name.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetGroupResult',
    'AwaitableGetGroupResult',
    'get_group',
    'get_group_output',
]

@pulumi.output_type
class GetGroupResult:
    """
    A collection of values returned by getGroup.
    """
    def __init__(__self__, acls=None, comment=None, group_id=None, id=None, members=None):
        if acls and not isinstance(acls, list):
            raise TypeError("Expected argument 'acls' to be a list")
        pulumi.set(__self__, "acls", acls)
        if comment and not isinstance(comment, str):
            raise TypeError("Expected argument 'comment' to be a str")
        pulumi.set(__self__, "comment", comment)
        if group_id and not isinstance(group_id, str):
            raise TypeError("Expected argument 'group_id' to be a str")
        pulumi.set(__self__, "group_id", group_id)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if members and not isinstance(members, list):
            raise TypeError("Expected argument 'members' to be a list")
        pulumi.set(__self__, "members", members)

    @property
    @pulumi.getter
    def acls(self) -> Sequence['outputs.GetGroupAclResult']:
        """
        The access control list.
        """
        return pulumi.get(self, "acls")

    @property
    @pulumi.getter
    def comment(self) -> str:
        """
        The group comment.
        """
        return pulumi.get(self, "comment")

    @property
    @pulumi.getter(name="groupId")
    def group_id(self) -> str:
        return pulumi.get(self, "group_id")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def members(self) -> Sequence[str]:
        """
        The group members as a list with `username@realm` entries.
        """
        return pulumi.get(self, "members")


class AwaitableGetGroupResult(GetGroupResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetGroupResult(
            acls=self.acls,
            comment=self.comment,
            group_id=self.group_id,
            id=self.id,
            members=self.members)


def get_group(group_id: Optional[str] = None,
              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetGroupResult:
    """
    Retrieves information about a specific user group.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    operations_team = proxmoxve.Permission.get_group(group_id="operations-team")
    ```


    :param str group_id: The group identifier.
    """
    __args__ = dict()
    __args__['groupId'] = group_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:Permission/getGroup:getGroup', __args__, opts=opts, typ=GetGroupResult).value

    return AwaitableGetGroupResult(
        acls=pulumi.get(__ret__, 'acls'),
        comment=pulumi.get(__ret__, 'comment'),
        group_id=pulumi.get(__ret__, 'group_id'),
        id=pulumi.get(__ret__, 'id'),
        members=pulumi.get(__ret__, 'members'))


@_utilities.lift_output_func(get_group)
def get_group_output(group_id: Optional[pulumi.Input[str]] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetGroupResult]:
    """
    Retrieves information about a specific user group.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    operations_team = proxmoxve.Permission.get_group(group_id="operations-team")
    ```


    :param str group_id: The group identifier.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from ._inputs import *

__all__ = ['ProviderArgs', 'Provider']

@pulumi.input_type
class ProviderArgs:
    def __init__(__self__, *,
                 api_token: Optional[pulumi.Input[str]] = None,
                 endpoint: Optional[pulumi.Input[str]] = None,
                 insecure: Optional[pulumi.Input[bool]] = None,
                 otp: Optional[pulumi.Input[str]] = None,
                 password: Optional[pulumi.Input[str]] = None,
                 ssh: Optional[pulumi.Input['ProviderSshArgs']] = None,
                 tmp_dir: Optional[pulumi.Input[str]] = None,
                 username: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a Provider resource.
        :param pulumi.Input[str] api_token: The API token for the Proxmox VE API.
        :param pulumi.Input[str] endpoint: The endpoint for the Proxmox VE API.
        :param pulumi.Input[bool] insecure: Whether to skip the TLS verification step.
        :param pulumi.Input[str] otp: The one-time password for the Proxmox VE API.
        :param pulumi.Input[str] password: The password for the Proxmox VE API.
        :param pulumi.Input['ProviderSshArgs'] ssh: The SSH configuration for the Proxmox nodes.
        :param pulumi.Input[str] tmp_dir: The alternative temporary directory.
        :param pulumi.Input[str] username: The username for the Proxmox VE API.
        """
        ProviderArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            api_token=api_token,
            endpoint=endpoint,
            insecure=insecure,
            otp=otp,
            password=password,
            ssh=ssh,
            tmp_dir=tmp_dir,
            username=username,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             api_token: Optional[pulumi.Input[str]] = None,
             endpoint: Optional[pulumi.Input[str]] = None,
             insecure: Optional[pulumi.Input[bool]] = None,
             otp: Optional[pulumi.Input[str]] = None,
             password: Optional[pulumi.Input[str]] = None,
             ssh: Optional[pulumi.Input['ProviderSshArgs']] = None,
             tmp_dir: Optional[pulumi.Input[str]] = None,
             username: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'apiToken' in kwargs:
            api_token = kwargs['apiToken']
        if 'tmpDir' in kwargs:
            tmp_dir = kwargs['tmpDir']

        if api_token is not None:
            _setter("api_token", api_token)
        if endpoint is not None:
            _setter("endpoint", endpoint)
        if insecure is not None:
            _setter("insecure", insecure)
        if otp is not None:
            warnings.warn("""The `otp` attribute is deprecated and will be removed in a future release. Please use the `api_token` attribute instead.""", DeprecationWarning)
            pulumi.log.warn("""otp is deprecated: The `otp` attribute is deprecated and will be removed in a future release. Please use the `api_token` attribute instead.""")
        if otp is not None:
            _setter("otp", otp)
        if password is not None:
            _setter("password", password)
        if ssh is not None:
            _setter("ssh", ssh)
        if tmp_dir is not None:
            _setter("tmp_dir", tmp_dir)
        if username is not None:
            _setter("username", username)

    @property
    @pulumi.getter(name="apiToken")
    def api_token(self) -> Optional[pulumi.Input[str]]:
        """
        The API token for the Proxmox VE API.
        """
        return pulumi.get(self, "api_token")

    @api_token.setter
    def api_token(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "api_token", value)

    @property
    @pulumi.getter
    def endpoint(self) -> Optional[pulumi.Input[str]]:
        """
        The endpoint for the Proxmox VE API.
        """
        return pulumi.get(self, "endpoint")

    @endpoint.setter
    def endpoint(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "endpoint", value)

    @property
    @pulumi.getter
    def insecure(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to skip the TLS verification step.
        """
        return pulumi.get(self, "insecure")

    @insecure.setter
    def insecure(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "insecure", value)

    @property
    @pulumi.getter
    def otp(self) -> Optional[pulumi.Input[str]]:
        """
        The one-time password for the Proxmox VE API.
        """
        warnings.warn("""The `otp` attribute is deprecated and will be removed in a future release. Please use the `api_token` attribute instead.""", DeprecationWarning)
        pulumi.log.warn("""otp is deprecated: The `otp` attribute is deprecated and will be removed in a future release. Please use the `api_token` attribute instead.""")

        return pulumi.get(self, "otp")

    @otp.setter
    def otp(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "otp", value)

    @property
    @pulumi.getter
    def password(self) -> Optional[pulumi.Input[str]]:
        """
        The password for the Proxmox VE API.
        """
        return pulumi.get(self, "password")

    @password.setter
    def password(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "password", value)

    @property
    @pulumi.getter
    def ssh(self) -> Optional[pulumi.Input['ProviderSshArgs']]:
        """
        The SSH configuration for the Proxmox nodes.
        """
        return pulumi.get(self, "ssh")

    @ssh.setter
    def ssh(self, value: Optional[pulumi.Input['ProviderSshArgs']]):
        pulumi.set(self, "ssh", value)

    @property
    @pulumi.getter(name="tmpDir")
    def tmp_dir(self) -> Optional[pulumi.Input[str]]:
        """
        The alternative temporary directory.
        """
        return pulumi.get(self, "tmp_dir")

    @tmp_dir.setter
    def tmp_dir(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "tmp_dir", value)

    @property
    @pulumi.getter
    def username(self) -> Optional[pulumi.Input[str]]:
        """
        The username for the Proxmox VE API.
        """
        return pulumi.get(self, "username")

    @username.setter
    def username(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "username", value)


class Provider(pulumi.ProviderResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 api_token: Optional[pulumi.Input[str]] = None,
                 endpoint: Optional[pulumi.Input[str]] = None,
                 insecure: Optional[pulumi.Input[bool]] = None,
                 otp: Optional[pulumi.Input[str]] = None,
                 password: Optional[pulumi.Input[str]] = None,
                 ssh: Optional[pulumi.Input[pulumi.InputType['ProviderSshArgs']]] = None,
                 tmp_dir: Optional[pulumi.Input[str]] = None,
                 username: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        The provider type for the proxmox package. By default, resources use package-wide configuration
        settings, however an explicit `Provider` instance may be created and passed during resource
        construction to achieve fine-grained programmatic control over provider settings. See the
        [documentation](https://www.pulumi.com/docs/reference/programming-model/#providers) for more information.

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] api_token: The API token for the Proxmox VE API.
        :param pulumi.Input[str] endpoint: The endpoint for the Proxmox VE API.
        :param pulumi.Input[bool] insecure: Whether to skip the TLS verification step.
        :param pulumi.Input[str] otp: The one-time password for the Proxmox VE API.
        :param pulumi.Input[str] password: The password for the Proxmox VE API.
        :param pulumi.Input[pulumi.InputType['ProviderSshArgs']] ssh: The SSH configuration for the Proxmox nodes.
        :param pulumi.Input[str] tmp_dir: The alternative temporary directory.
        :param pulumi.Input[str] username: The username for the Proxmox VE API.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[ProviderArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        The provider type for the proxmox package. By default, resources use package-wide configuration
        settings, however an explicit `Provider` instance may be created and passed during resource
        construction to achieve fine-grained programmatic control over provider settings. See the
        [documentation](https://www.pulumi.com/docs/reference/programming-model/#providers) for more information.

        :param str resource_name: The name of the resource.
        :param ProviderArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ProviderArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            ProviderArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 api_token: Optional[pulumi.Input[str]] = None,
                 endpoint: Optional[pulumi.Input[str]] = None,
                 insecure: Optional[pulumi.Input[bool]] = None,
                 otp: Optional[pulumi.Input[str]] = None,
                 password: Optional[pulumi.Input[str]] = None,
                 ssh: Optional[pulumi.Input[pulumi.InputType['ProviderSshArgs']]] = None,
                 tmp_dir: Optional[pulumi.Input[str]] = None,
                 username: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ProviderArgs.__new__(ProviderArgs)

            __props__.__dict__["api_token"] = None if api_token is None else pulumi.Output.secret(api_token)
            __props__.__dict__["endpoint"] = endpoint
            __props__.__dict__["insecure"] = pulumi.Output.from_input(insecure).apply(pulumi.runtime.to_json) if insecure is not None else None
            __props__.__dict__["otp"] = otp
            __props__.__dict__["password"] = None if password is None else pulumi.Output.secret(password)
            if ssh is not None and not isinstance(ssh, ProviderSshArgs):
                ssh = ssh or {}
                def _setter(key, value):
                    ssh[key] = value
                ProviderSshArgs._configure(_setter, **ssh)
            __props__.__dict__["ssh"] = pulumi.Output.from_input(ssh).apply(pulumi.runtime.to_json) if ssh is not None else None
            __props__.__dict__["tmp_dir"] = tmp_dir
            __props__.__dict__["username"] = username
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["apiToken", "password"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(Provider, __self__).__init__(
            'proxmoxve',
            resource_name,
            __props__,
            opts)

    @property
    @pulumi.getter(name="apiToken")
    def api_token(self) -> pulumi.Output[Optional[str]]:
        """
        The API token for the Proxmox VE API.
        """
        return pulumi.get(self, "api_token")

    @property
    @pulumi.getter
    def endpoint(self) -> pulumi.Output[Optional[str]]:
        """
        The endpoint for the Proxmox VE API.
        """
        return pulumi.get(self, "endpoint")

    @property
    @pulumi.getter
    def otp(self) -> pulumi.Output[Optional[str]]:
        """
        The one-time password for the Proxmox VE API.
        """
        warnings.warn("""The `otp` attribute is deprecated and will be removed in a future release. Please use the `api_token` attribute instead.""", DeprecationWarning)
        pulumi.log.warn("""otp is deprecated: The `otp` attribute is deprecated and will be removed in a future release. Please use the `api_token` attribute instead.""")

        return pulumi.get(self, "otp")

    @property
    @pulumi.getter
    def password(self) -> pulumi.Output[Optional[str]]:
        """
        The password for the Proxmox VE API.
        """
        return pulumi.get(self, "password")

    @property
    @pulumi.getter(name="tmpDir")
    def tmp_dir(self) -> pulumi.Output[Optional[str]]:
        """
        The alternative temporary directory.
        """
        return pulumi.get(self, "tmp_dir")

    @property
    @pulumi.getter
    def username(self) -> pulumi.Output[Optional[str]]:
        """
        The username for the Proxmox VE API.
        """
        return pulumi.get(self, "username")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'GetVirtualMachinesResult',
    'AwaitableGetVirtualMachinesResult',
    'get_virtual_machines',
    'get_virtual_machines_output',
]

@pulumi.output_type
class GetVirtualMachinesResult:
    """
    A collection of values returned by getVirtualMachines.
    """
    def __init__(__self__, id=None, node_name=None, tags=None, vms=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if node_name and not isinstance(node_name, str):
            raise TypeError("Expected argument 'node_name' to be a str")
        pulumi.set(__self__, "node_name", node_name)
        if tags and not isinstance(tags, list):
            raise TypeError("Expected argument 'tags' to be a list")
        pulumi.set(__self__, "tags", tags)
        if vms and not isinstance(vms, list):
            raise TypeError("Expected argument 'vms' to be a list")
        pulumi.set(__self__, "vms", vms)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[str]:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def tags(self) -> Optional[Sequence[str]]:
        """
        A list of tags of the VM.
        """
        return pulumi.get(self, "tags")

    @property
    @pulumi.getter
    def vms(self) -> Sequence['outputs.GetVirtualMachinesVmResult']:
        """
        The VMs list.
        """
        return pulumi.get(self, "vms")


class AwaitableGetVirtualMachinesResult(GetVirtualMachinesResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetVirtualMachinesResult(
            id=self.id,
            node_name=self.node_name,
            tags=self.tags,
            vms=self.vms)


def get_virtual_machines(node_name: Optional[str] = None,
                         tags: Optional[Sequence[str]] = None,
                         opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetVirtualMachinesResult:
    """
    Retrieves information about all VMs in the Proxmox cluster.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    ubuntu_vms = proxmoxve.VM.get_virtual_machines(tags=["ubuntu"])
    ```


    :param str node_name: The node name.
    :param Sequence[str] tags: A list of tags to filter the VMs. The VM must have all
           the tags to be included in the result.
    """
    __args__ = dict()
    __args__['nodeName'] = node_name
    __args__['tags'] = tags
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:VM/getVirtualMachines:getVirtualMachines', __args__, opts=opts, typ=GetVirtualMachinesResult).value

    return AwaitableGetVirtualMachinesResult(
        id=pulumi.get(__ret__, 'id'),
        node_name=pulumi.get(__ret__, 'node_name'),
        tags=pulumi.get(__ret__, 'tags'),
        vms=pulumi.get(__ret__, 'vms'))


@_utilities.lift_output_func(get_virtual_machines)
def get_virtual_machines_output(node_name: Optional[pulumi.Input[Optional[str]]] = None,
                                tags: Optional[pulumi.Input[Optional[Sequence[str]]]] = None,
                                opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetVirtualMachinesResult]:
    """
    Retrieves information about all VMs in the Proxmox cluster.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    ubuntu_vms = proxmoxve.VM.get_virtual_machines(tags=["ubuntu"])
    ```


    :param str node_name: The node name.
    :param Sequence[str] tags: A list of tags to filter the VMs. The VM must have all
           the tags to be included in the result.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs

__all__ = [
    'GetHostsResult',
    'AwaitableGetHostsResult',
    'get_hosts',
    'get_hosts_output',
]

@pulumi.output_type
class GetHostsResult:
    """
    A collection of values returned by getHosts.
    """
    def __init__(__self__, addresses=None, digest=None, entries=None, hostnames=None, id=None, node_name=None):
        if addresses and not isinstance(addresses, list):
            raise TypeError("Expected argument 'addresses' to be a list")
        pulumi.set(__self__, "addresses", addresses)
        if digest and not isinstance(digest, str):
            raise TypeError("Expected argument 'digest' to be a str")
        pulumi.set(__self__, "digest", digest)
        if entries and not isinstance(entries, list):
            raise TypeError("Expected argument 'entries' to be a list")
        pulumi.set(__self__, "entries", entries)
        if hostnames and not isinstance(hostnames, list):
            raise TypeError("Expected argument 'hostnames' to be a list")
        pulumi.set(__self__, "hostnames", hostnames)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if node_name and not isinstance(node_name, str):
            raise TypeError("Expected argument 'node_name' to be a str")
        pulumi.set(__self__, "node_name", node_name)

    @property
    @pulumi.getter
    def addresses(self) -> Sequence[str]:
        """
        The IP addresses.
        """
        return pulumi.get(self, "addresses")

    @property
    @pulumi.getter
    def digest(self) -> str:
        """
        The SHA1 digest.
        """
        return pulumi.get(self, "digest")

    @property
    @pulumi.getter
    def entries(self) -> Sequence['outputs.GetHostsEntryResult']:
        """
        The host entries (conversion of `addresses` and `hostnames` into
        objects).
        """
        return pulumi.get(self, "entries")

    @property
    @pulumi.getter
    def hostnames(self) -> Sequence[Sequence[str]]:
        """
        The hostnames associated with each of the IP addresses.
        """
        return pulumi.get(self, "hostnames")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> str:
        return pulumi.get(self, "node_name")


class AwaitableGetHostsResult(GetHostsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetHostsResult(
            addresses=self.addresses,
            digest=self.digest,
            entries=self.entries,
            hostnames=self.hostnames,
            id=self.id,
            node_name=self.node_name)


def get_hosts(node_name: Optional[str] = None,
              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetHostsResult:
    """
    Retrieves all the host entries from a specific node.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    first_node_host_entries = proxmoxve.get_hosts(node_name="first-node")
    ```


    :param str node_name: A node name.
    """
    __args__ = dict()
    __args__['nodeName'] = node_name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:index/getHosts:getHosts', __args__, opts=opts, typ=GetHostsResult).value

    return AwaitableGetHostsResult(
        addresses=pulumi.get(__ret__, 'addresses'),
        digest=pulumi.get(__ret__, 'digest'),
        entries=pulumi.get(__ret__, 'entries'),
        hostnames=pulumi.get(__ret__, 'hostnames'),
        id=pulumi.get(__ret__, 'id'),
        node_name=pulumi.get(__ret__, 'node_name'))


@_utilities.lift_output_func(get_hosts)
def get_hosts_output(node_name: Optional[pulumi.Input[str]] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetHostsResult]:
    """
    Retrieves all the host entries from a specific node.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    first_node_host_entries = proxmoxve.get_hosts(node_name="first-node")
    ```


    :param str node_name: A node name.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetPoolsResult',
    'AwaitableGetPoolsResult',
    'get_pools',
    'get_pools_output',
]

@pulumi.output_type
class GetPoolsResult:
    """
    A collection of values returned by getPools.
    """
    def __init__(__self__, id=None, pool_ids=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if pool_ids and not isinstance(pool_ids, list):
            raise TypeError("Expected argument 'pool_ids' to be a list")
        pulumi.set(__self__, "pool_ids", pool_ids)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="poolIds")
    def pool_ids(self) -> Sequence[str]:
        """
        The pool identifiers.
        """
        return pulumi.get(self, "pool_ids")


class AwaitableGetPoolsResult(GetPoolsResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetPoolsResult(
            id=self.id,
            pool_ids=self.pool_ids)


def get_pools(opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetPoolsResult:
    """
    Retrieves the identifiers for all the available resource pools.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    available_pools = proxmoxve.Permission.get_pools()
    ```
    """
    __args__ = dict()
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:Permission/getPools:getPools', __args__, opts=opts, typ=GetPoolsResult).value

    return AwaitableGetPoolsResult(
        id=pulumi.get(__ret__, 'id'),
        pool_ids=pulumi.get(__ret__, 'pool_ids'))


@_utilities.lift_output_func(get_pools)
def get_pools_output(opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetPoolsResult]:
    """
    Retrieves the identifiers for all the available resource pools.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    available_pools = proxmoxve.Permission.get_pools()
    ```
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'FileSourceFileArgs',
    'FileSourceRawArgs',
]

@pulumi.input_type
class FileSourceFileArgs:
    def __init__(__self__, *,
                 path: pulumi.Input[str],
                 changed: Optional[pulumi.Input[bool]] = None,
                 checksum: Optional[pulumi.Input[str]] = None,
                 file_name: Optional[pulumi.Input[str]] = None,
                 insecure: Optional[pulumi.Input[bool]] = None):
        """
        :param pulumi.Input[str] path: A path to a local file or a URL.
        :param pulumi.Input[str] checksum: The SHA256 checksum of the source file.
        :param pulumi.Input[str] file_name: The file name.
        :param pulumi.Input[bool] insecure: Whether to skip the TLS verification step for
               HTTPS sources (defaults to `false`).
        """
        FileSourceFileArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            path=path,
            changed=changed,
            checksum=checksum,
            file_name=file_name,
            insecure=insecure,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             path: pulumi.Input[str],
             changed: Optional[pulumi.Input[bool]] = None,
             checksum: Optional[pulumi.Input[str]] = None,
             file_name: Optional[pulumi.Input[str]] = None,
             insecure: Optional[pulumi.Input[bool]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'fileName' in kwargs:
            file_name = kwargs['fileName']

        _setter("path", path)
        if changed is not None:
            _setter("changed", changed)
        if checksum is not None:
            _setter("checksum", checksum)
        if file_name is not None:
            _setter("file_name", file_name)
        if insecure is not None:
            _setter("insecure", insecure)

    @property
    @pulumi.getter
    def path(self) -> pulumi.Input[str]:
        """
        A path to a local file or a URL.
        """
        return pulumi.get(self, "path")

    @path.setter
    def path(self, value: pulumi.Input[str]):
        pulumi.set(self, "path", value)

    @property
    @pulumi.getter
    def changed(self) -> Optional[pulumi.Input[bool]]:
        return pulumi.get(self, "changed")

    @changed.setter
    def changed(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "changed", value)

    @property
    @pulumi.getter
    def checksum(self) -> Optional[pulumi.Input[str]]:
        """
        The SHA256 checksum of the source file.
        """
        return pulumi.get(self, "checksum")

    @checksum.setter
    def checksum(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "checksum", value)

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> Optional[pulumi.Input[str]]:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @file_name.setter
    def file_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "file_name", value)

    @property
    @pulumi.getter
    def insecure(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to skip the TLS verification step for
        HTTPS sources (defaults to `false`).
        """
        return pulumi.get(self, "insecure")

    @insecure.setter
    def insecure(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "insecure", value)


@pulumi.input_type
class FileSourceRawArgs:
    def __init__(__self__, *,
                 data: pulumi.Input[str],
                 file_name: pulumi.Input[str],
                 resize: Optional[pulumi.Input[int]] = None):
        """
        :param pulumi.Input[str] data: The raw data.
        :param pulumi.Input[str] file_name: The file name.
        :param pulumi.Input[int] resize: The number of bytes to resize the file to.
        """
        FileSourceRawArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            data=data,
            file_name=file_name,
            resize=resize,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             data: pulumi.Input[str],
             file_name: pulumi.Input[str],
             resize: Optional[pulumi.Input[int]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'fileName' in kwargs:
            file_name = kwargs['fileName']

        _setter("data", data)
        _setter("file_name", file_name)
        if resize is not None:
            _setter("resize", resize)

    @property
    @pulumi.getter
    def data(self) -> pulumi.Input[str]:
        """
        The raw data.
        """
        return pulumi.get(self, "data")

    @data.setter
    def data(self, value: pulumi.Input[str]):
        pulumi.set(self, "data", value)

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> pulumi.Input[str]:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @file_name.setter
    def file_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "file_name", value)

    @property
    @pulumi.getter
    def resize(self) -> Optional[pulumi.Input[int]]:
        """
        The number of bytes to resize the file to.
        """
        return pulumi.get(self, "resize")

    @resize.setter
    def resize(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "resize", value)





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from . import _utilities
import typing
# Export this package's modules as members:
from .certifi import *
from .dns import *
from .get_dns import *
from .get_hosts import *
from .get_time import *
from .hosts import *
from .provider import *
from .time import *
from ._inputs import *
from . import outputs

# Make subpackages available:
if typing.TYPE_CHECKING:
    import pulumi_proxmoxve.cluster as __cluster
    cluster = __cluster
    import pulumi_proxmoxve.config as __config
    config = __config
    import pulumi_proxmoxve.ct as __ct
    ct = __ct
    import pulumi_proxmoxve.network as __network
    network = __network
    import pulumi_proxmoxve.permission as __permission
    permission = __permission
    import pulumi_proxmoxve.storage as __storage
    storage = __storage
    import pulumi_proxmoxve.vm as __vm
    vm = __vm
else:
    cluster = _utilities.lazy_import('pulumi_proxmoxve.cluster')
    config = _utilities.lazy_import('pulumi_proxmoxve.config')
    ct = _utilities.lazy_import('pulumi_proxmoxve.ct')
    network = _utilities.lazy_import('pulumi_proxmoxve.network')
    permission = _utilities.lazy_import('pulumi_proxmoxve.permission')
    storage = _utilities.lazy_import('pulumi_proxmoxve.storage')
    vm = _utilities.lazy_import('pulumi_proxmoxve.vm')

_utilities.register(
    resource_modules="""
[
 {
  "pkg": "proxmoxve",
  "mod": "CT/container",
  "fqn": "pulumi_proxmoxve.ct",
  "classes": {
   "proxmoxve:CT/container:Container": "Container"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Network/firewall",
  "fqn": "pulumi_proxmoxve.network",
  "classes": {
   "proxmoxve:Network/firewall:Firewall": "Firewall"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Network/firewallAlias",
  "fqn": "pulumi_proxmoxve.network",
  "classes": {
   "proxmoxve:Network/firewallAlias:FirewallAlias": "FirewallAlias"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Network/firewallIPSet",
  "fqn": "pulumi_proxmoxve.network",
  "classes": {
   "proxmoxve:Network/firewallIPSet:FirewallIPSet": "FirewallIPSet"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Network/firewallOptions",
  "fqn": "pulumi_proxmoxve.network",
  "classes": {
   "proxmoxve:Network/firewallOptions:FirewallOptions": "FirewallOptions"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Network/firewallRules",
  "fqn": "pulumi_proxmoxve.network",
  "classes": {
   "proxmoxve:Network/firewallRules:FirewallRules": "FirewallRules"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Network/firewallSecurityGroup",
  "fqn": "pulumi_proxmoxve.network",
  "classes": {
   "proxmoxve:Network/firewallSecurityGroup:FirewallSecurityGroup": "FirewallSecurityGroup"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Permission/group",
  "fqn": "pulumi_proxmoxve.permission",
  "classes": {
   "proxmoxve:Permission/group:Group": "Group"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Permission/pool",
  "fqn": "pulumi_proxmoxve.permission",
  "classes": {
   "proxmoxve:Permission/pool:Pool": "Pool"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Permission/role",
  "fqn": "pulumi_proxmoxve.permission",
  "classes": {
   "proxmoxve:Permission/role:Role": "Role"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Permission/user",
  "fqn": "pulumi_proxmoxve.permission",
  "classes": {
   "proxmoxve:Permission/user:User": "User"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "Storage/file",
  "fqn": "pulumi_proxmoxve.storage",
  "classes": {
   "proxmoxve:Storage/file:File": "File"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "VM/virtualMachine",
  "fqn": "pulumi_proxmoxve.vm",
  "classes": {
   "proxmoxve:VM/virtualMachine:VirtualMachine": "VirtualMachine"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "index/certifi",
  "fqn": "pulumi_proxmoxve",
  "classes": {
   "proxmoxve:index/certifi:Certifi": "Certifi"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "index/dNS",
  "fqn": "pulumi_proxmoxve",
  "classes": {
   "proxmoxve:index/dNS:DNS": "DNS"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "index/hosts",
  "fqn": "pulumi_proxmoxve",
  "classes": {
   "proxmoxve:index/hosts:Hosts": "Hosts"
  }
 },
 {
  "pkg": "proxmoxve",
  "mod": "index/time",
  "fqn": "pulumi_proxmoxve",
  "classes": {
   "proxmoxve:index/time:Time": "Time"
  }
 }
]
""",
    resource_packages="""
[
 {
  "pkg": "proxmoxve",
  "token": "pulumi:providers:proxmoxve",
  "fqn": "pulumi_proxmoxve",
  "class": "Provider"
 }
]
"""
)



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'FileSourceFile',
    'FileSourceRaw',
]

@pulumi.output_type
class FileSourceFile(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "fileName":
            suggest = "file_name"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in FileSourceFile. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        FileSourceFile.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        FileSourceFile.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 path: str,
                 changed: Optional[bool] = None,
                 checksum: Optional[str] = None,
                 file_name: Optional[str] = None,
                 insecure: Optional[bool] = None):
        """
        :param str path: A path to a local file or a URL.
        :param str checksum: The SHA256 checksum of the source file.
        :param str file_name: The file name.
        :param bool insecure: Whether to skip the TLS verification step for
               HTTPS sources (defaults to `false`).
        """
        FileSourceFile._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            path=path,
            changed=changed,
            checksum=checksum,
            file_name=file_name,
            insecure=insecure,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             path: str,
             changed: Optional[bool] = None,
             checksum: Optional[str] = None,
             file_name: Optional[str] = None,
             insecure: Optional[bool] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'fileName' in kwargs:
            file_name = kwargs['fileName']

        _setter("path", path)
        if changed is not None:
            _setter("changed", changed)
        if checksum is not None:
            _setter("checksum", checksum)
        if file_name is not None:
            _setter("file_name", file_name)
        if insecure is not None:
            _setter("insecure", insecure)

    @property
    @pulumi.getter
    def path(self) -> str:
        """
        A path to a local file or a URL.
        """
        return pulumi.get(self, "path")

    @property
    @pulumi.getter
    def changed(self) -> Optional[bool]:
        return pulumi.get(self, "changed")

    @property
    @pulumi.getter
    def checksum(self) -> Optional[str]:
        """
        The SHA256 checksum of the source file.
        """
        return pulumi.get(self, "checksum")

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> Optional[str]:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @property
    @pulumi.getter
    def insecure(self) -> Optional[bool]:
        """
        Whether to skip the TLS verification step for
        HTTPS sources (defaults to `false`).
        """
        return pulumi.get(self, "insecure")


@pulumi.output_type
class FileSourceRaw(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "fileName":
            suggest = "file_name"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in FileSourceRaw. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        FileSourceRaw.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        FileSourceRaw.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 data: str,
                 file_name: str,
                 resize: Optional[int] = None):
        """
        :param str data: The raw data.
        :param str file_name: The file name.
        :param int resize: The number of bytes to resize the file to.
        """
        FileSourceRaw._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            data=data,
            file_name=file_name,
            resize=resize,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             data: str,
             file_name: str,
             resize: Optional[int] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'fileName' in kwargs:
            file_name = kwargs['fileName']

        _setter("data", data)
        _setter("file_name", file_name)
        if resize is not None:
            _setter("resize", resize)

    @property
    @pulumi.getter
    def data(self) -> str:
        """
        The raw data.
        """
        return pulumi.get(self, "data")

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> str:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @property
    @pulumi.getter
    def resize(self) -> Optional[int]:
        """
        The number of bytes to resize the file to.
        """
        return pulumi.get(self, "resize")





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GetDatastoresResult',
    'AwaitableGetDatastoresResult',
    'get_datastores',
    'get_datastores_output',
]

@pulumi.output_type
class GetDatastoresResult:
    """
    A collection of values returned by getDatastores.
    """
    def __init__(__self__, actives=None, content_types=None, datastore_ids=None, enableds=None, id=None, node_name=None, shareds=None, space_availables=None, space_totals=None, space_useds=None, types=None):
        if actives and not isinstance(actives, list):
            raise TypeError("Expected argument 'actives' to be a list")
        pulumi.set(__self__, "actives", actives)
        if content_types and not isinstance(content_types, list):
            raise TypeError("Expected argument 'content_types' to be a list")
        pulumi.set(__self__, "content_types", content_types)
        if datastore_ids and not isinstance(datastore_ids, list):
            raise TypeError("Expected argument 'datastore_ids' to be a list")
        pulumi.set(__self__, "datastore_ids", datastore_ids)
        if enableds and not isinstance(enableds, list):
            raise TypeError("Expected argument 'enableds' to be a list")
        pulumi.set(__self__, "enableds", enableds)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if node_name and not isinstance(node_name, str):
            raise TypeError("Expected argument 'node_name' to be a str")
        pulumi.set(__self__, "node_name", node_name)
        if shareds and not isinstance(shareds, list):
            raise TypeError("Expected argument 'shareds' to be a list")
        pulumi.set(__self__, "shareds", shareds)
        if space_availables and not isinstance(space_availables, list):
            raise TypeError("Expected argument 'space_availables' to be a list")
        pulumi.set(__self__, "space_availables", space_availables)
        if space_totals and not isinstance(space_totals, list):
            raise TypeError("Expected argument 'space_totals' to be a list")
        pulumi.set(__self__, "space_totals", space_totals)
        if space_useds and not isinstance(space_useds, list):
            raise TypeError("Expected argument 'space_useds' to be a list")
        pulumi.set(__self__, "space_useds", space_useds)
        if types and not isinstance(types, list):
            raise TypeError("Expected argument 'types' to be a list")
        pulumi.set(__self__, "types", types)

    @property
    @pulumi.getter
    def actives(self) -> Sequence[bool]:
        """
        Whether the datastore is active.
        """
        return pulumi.get(self, "actives")

    @property
    @pulumi.getter(name="contentTypes")
    def content_types(self) -> Sequence[Sequence[str]]:
        """
        The allowed content types.
        """
        return pulumi.get(self, "content_types")

    @property
    @pulumi.getter(name="datastoreIds")
    def datastore_ids(self) -> Sequence[str]:
        """
        The datastore identifiers.
        """
        return pulumi.get(self, "datastore_ids")

    @property
    @pulumi.getter
    def enableds(self) -> Sequence[bool]:
        """
        Whether the datastore is enabled.
        """
        return pulumi.get(self, "enableds")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> str:
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def shareds(self) -> Sequence[bool]:
        """
        Whether the datastore is shared.
        """
        return pulumi.get(self, "shareds")

    @property
    @pulumi.getter(name="spaceAvailables")
    def space_availables(self) -> Sequence[int]:
        """
        The available space in bytes.
        """
        return pulumi.get(self, "space_availables")

    @property
    @pulumi.getter(name="spaceTotals")
    def space_totals(self) -> Sequence[int]:
        """
        The total space in bytes.
        """
        return pulumi.get(self, "space_totals")

    @property
    @pulumi.getter(name="spaceUseds")
    def space_useds(self) -> Sequence[int]:
        """
        The used space in bytes.
        """
        return pulumi.get(self, "space_useds")

    @property
    @pulumi.getter
    def types(self) -> Sequence[str]:
        """
        The storage types.
        """
        return pulumi.get(self, "types")


class AwaitableGetDatastoresResult(GetDatastoresResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDatastoresResult(
            actives=self.actives,
            content_types=self.content_types,
            datastore_ids=self.datastore_ids,
            enableds=self.enableds,
            id=self.id,
            node_name=self.node_name,
            shareds=self.shareds,
            space_availables=self.space_availables,
            space_totals=self.space_totals,
            space_useds=self.space_useds,
            types=self.types)


def get_datastores(node_name: Optional[str] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetDatastoresResult:
    """
    Retrieves information about all the datastores available to a specific node.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    first_node = proxmoxve.Storage.get_datastores(node_name="first-node")
    ```


    :param str node_name: A node name.
    """
    __args__ = dict()
    __args__['nodeName'] = node_name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:Storage/getDatastores:getDatastores', __args__, opts=opts, typ=GetDatastoresResult).value

    return AwaitableGetDatastoresResult(
        actives=pulumi.get(__ret__, 'actives'),
        content_types=pulumi.get(__ret__, 'content_types'),
        datastore_ids=pulumi.get(__ret__, 'datastore_ids'),
        enableds=pulumi.get(__ret__, 'enableds'),
        id=pulumi.get(__ret__, 'id'),
        node_name=pulumi.get(__ret__, 'node_name'),
        shareds=pulumi.get(__ret__, 'shareds'),
        space_availables=pulumi.get(__ret__, 'space_availables'),
        space_totals=pulumi.get(__ret__, 'space_totals'),
        space_useds=pulumi.get(__ret__, 'space_useds'),
        types=pulumi.get(__ret__, 'types'))


@_utilities.lift_output_func(get_datastores)
def get_datastores_output(node_name: Optional[pulumi.Input[str]] = None,
                          opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetDatastoresResult]:
    """
    Retrieves information about all the datastores available to a specific node.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    first_node = proxmoxve.Storage.get_datastores(node_name="first-node")
    ```


    :param str node_name: A node name.
    """
    ...
